package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.concurrent.CountDownLatch;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 14:24
 */
public abstract class MultipleCallExperimentTask extends ExperimentTask {
    private static final Logger LOG = LoggerFactory.getLogger(MultipleCallExperimentTask.class);

    protected final HighLevelTaskFactory hltf;
    private final ObservableList<TaskExecution> taskExecutions = FXCollections.observableArrayList();

    private boolean finished = false;

    MultipleCallExperimentTask(Experiment experiment, HighLevelTaskFactory hltf) {
        super(experiment);
        this.hltf = hltf;
    }

    @Override
    protected final TaskStatusIndicator.Status executeTask() {
        LOG.trace("Starting execution of {} for {}", getClass().getSimpleName(), experiment.getName());

        onTaskStarting();

        Collection<Task> tasks = createTasks();

        CountDownLatch latch = new CountDownLatch(tasks.size());

        for (Task task : tasks) {

            TaskExecution te = hltf.submitTask(task, (task1, taskExecution1, state) -> {
                registerFinishedTaskExecution(taskExecution1);
                onTaskFinished(task1, taskExecution1);
                LOG.trace("Finished execution of {} for {}", getClass().getSimpleName(), experiment.getName());

                latch.countDown();

            });

            taskExecutions.add(te);
        }

        try {
            latch.await();
        } catch (InterruptedException ignored) {
        }

        finished = true;
        LOG.trace("Finished execution of all tasks of {} for {}", getClass().getSimpleName(),experiment.getName());



        TaskStatusIndicator.Status endStatus = TaskStatusIndicator.Status.SUCCESS;

        for (TaskExecution taskExecution : taskExecutions) {
            if(taskExecution.getState() == TaskExecution.TaskState.FAILED)
                endStatus = TaskStatusIndicator.Status.FAILED;
            else if(taskExecution.getState() != TaskExecution.TaskState.SUCCESS)
                throw new IllegalStateException("Did not expected state " + taskExecution.getState().name()
                        + " for finished task " + taskExecution.getTask().toString());
        }

        return endStatus;
    }

    public void onTaskStarting() {

    }

    public boolean isFinished() {
        return finished;
    }

    public void onTaskFinished(Task task, TaskExecution taskExecution) {
    }

    public abstract Collection<Task> createTasks();

    public ObservableList<TaskExecution> getTaskExecutions() {
        return taskExecutions;
    }

    @Override
    public String toString() {
        return getClass().getSimpleName();
    }
}
